@@ -88,8 +88,8 @@ func Run(v *viper.Viper, statz *stats.Stats, cmd *cobra.Command, paths []string)
 
 		// ensure db is closed after we're finished
 		defer func() {
-			if err := db.Close(); err != nil {
-				log.Errorf("failed to close cache: %v", err)
+			if closeErr := db.Close(); closeErr != nil {
+				log.Errorf("failed to close cache: %v", closeErr)
 			}
 		}()
 	}
@@ -126,30 +126,8 @@ func Run(v *viper.Viper, statz *stats.Stats, cmd *cobra.Command, paths []string)
 		return errors.New("exactly one path should be specified when using the --stdin flag")
 	}
 
-	// checks all paths are contained within the tree root and exist
-	// also "normalize" paths so they're relative to cfg.TreeRoot
-	for i, path := range paths {
-		absolutePath, err := filepath.Abs(path)
-		if err != nil {
-			return fmt.Errorf("error computing absolute path of %s: %w", path, err)
-		}
-
-		relativePath, err := filepath.Rel(cfg.TreeRoot, absolutePath)
-		if err != nil {
-			return fmt.Errorf("error computing relative path from %s to %s: %w", cfg.TreeRoot, absolutePath, err)
-		}
-
-		if strings.HasPrefix(relativePath, "..") {
-			return fmt.Errorf("path %s not inside the tree root %s", path, cfg.TreeRoot)
-		}
-
-		paths[i] = relativePath
-
-		if walkType != walk.Stdin {
-			if _, err = os.Stat(absolutePath); err != nil {
-				return fmt.Errorf("path %s not found", path)
-			}
-		}
+	if err = resolvePaths(paths, walkType, cfg.TreeRoot); err != nil {
+		return err
 	}
 
 	// create a composite formatter which will handle applying the correct formatters to each file we traverse
@@ -167,48 +145,110 @@ func Run(v *viper.Viper, statz *stats.Stats, cmd *cobra.Command, paths []string)
 	// start traversing
 	files := make([]*walk.File, BatchSize)
 
+	var (
+		n                  int
+		readErr, formatErr error
+	)
+
 	for {
 		// read the next batch
-		readCtx, cancel := context.WithTimeout(ctx, 1*time.Second)
-		n, err := walker.Read(readCtx, files)
+		readCtx, cancelRead := context.WithTimeout(ctx, 1*time.Second)
+
+		n, readErr = walker.Read(readCtx, files)
+		log.Debugf("read %d files", n)
 
 		// ensure context is cancelled to release resources
-		cancel()
+		cancelRead()
 
-		// format
-		if err := formatter.Apply(ctx, files[:n]); err != nil {
-			return fmt.Errorf("formatting failure: %w", err)
+		// format any files that were read before processing the read error
+		if formatErr = formatter.Apply(ctx, files[:n]); formatErr != nil {
+			break
 		}
 
-		if errors.Is(err, io.EOF) {
-			// we have finished traversing
+		// stop reading files if there was a read error
+		if readErr != nil {
 			break
-		} else if err != nil {
-			// something went wrong
-			return fmt.Errorf("failed to read files: %w", err)
 		}
 	}
 
-	// finalize formatting
-	formatErr := formatter.Close(ctx)
+	// finalize formatting (there could be formatting tasks in-flight)
+	formatCloseErr := formatter.Close(ctx)
 
 	// close the walker, ensuring any pending file release hooks finish
-	if err = walker.Close(); err != nil {
-		return fmt.Errorf("failed to close walker: %w", err)
+	walkerCloseErr := walker.Close()
+
+	// print stats to stderr
+	if !cfg.Quiet {
+		statz.PrintToStderr()
 	}
 
-	// print stats to stdout, unless we are processing from stdin and therefore outputting the results to stdout
-	if !cfg.Stdin {
-		statz.Print()
+	// process errors
+
+	//nolint:gocritic
+	if errors.Is(readErr, io.EOF) {
+		// nothing more to read, reset the error and break out of the read loop
+		log.Debugf("no more files to read")
+	} else if errors.Is(readErr, context.DeadlineExceeded) {
+		// the read timed-out
+		return errors.New("timeout reading files")
+	} else if readErr != nil {
+		// something unexpected happened
+		return fmt.Errorf("failed to read files: %w", readErr)
 	}
 
 	if formatErr != nil {
-		// return an error if any formatting failures were detected
-		return formatErr //nolint:wrapcheck
-	} else if cfg.FailOnChange && statz.Value(stats.Changed) != 0 {
+		return fmt.Errorf("failed to format files: %w", formatErr)
+	}
+
+	if formatCloseErr != nil {
+		return fmt.Errorf("failed to finalise formatting: %w", formatCloseErr)
+	}
+
+	if walkerCloseErr != nil {
+		return fmt.Errorf("failed to close walker: %w", walkerCloseErr)
+	}
+
+	if cfg.FailOnChange && statz.Value(stats.Changed) != 0 {
 		// if fail on change has been enabled, check that no files were actually changed, throwing an error if so
 		return ErrFailOnChange
 	}
 
 	return nil
 }
+
+// resolvePaths checks all paths are contained within the tree root and exist
+// also "normalize" paths so they're relative to `cfg.TreeRoot`
+// Symlinks are allowed in `paths` and we resolve them here, since
+// the readers will ignore symlinks.
+func resolvePaths(paths []string, walkType walk.Type, treeRoot string) error {
+	for i, path := range paths {
+		log.Debugf("Resolving path '%s': %v", path, walkType)
+
+		absolutePath, err := filepath.Abs(path)
+		if err != nil {
+			return fmt.Errorf("error computing absolute path of %s: %w", path, err)
+		}
+
+		if walkType != walk.Stdin {
+			realPath, err := filepath.EvalSymlinks(absolutePath)
+			if err != nil {
+				return fmt.Errorf("path %s not found: %w", absolutePath, err)
+			}
+
+			absolutePath = realPath
+		}
+
+		relativePath, err := filepath.Rel(treeRoot, absolutePath)
+		if err != nil {
+			return fmt.Errorf("error computing relative path from %s to %s: %w", treeRoot, absolutePath, err)
+		}
+
+		if strings.HasPrefix(relativePath, "..") {
+			return fmt.Errorf("path %s not inside the tree root %s", path, treeRoot)
+		}
+
+		paths[i] = relativePath
+	}
+
+	return nil
+}
