@@ -72,14 +72,14 @@ func TestOnUnmatched(t *testing.T) {
 		}
 	}
 
-	// default is WARN
+	// default is INFO
 	t.Run("default", func(t *testing.T) {
-		treefmt(t, withNoError(t), withOutput(checkOutput(log.WarnLevel)))
+		treefmt(t, withArgs("-v"), withNoError(t), withStderr(checkOutput(log.InfoLevel)))
 	})
 
 	// should exit with error when using fatal
 	t.Run("fatal", func(t *testing.T) {
-		errorFn := func(err error) {
+		errorFn := func(as *require.Assertions, err error) {
 			as.ErrorContains(err, "no formatter for path: "+expectedPaths[0])
 		}
 
@@ -99,7 +99,7 @@ func TestOnUnmatched(t *testing.T) {
 			treefmt(t,
 				withArgs("-vv", "--on-unmatched", levelStr),
 				withNoError(t),
-				withOutput(checkOutput(level)),
+				withStderr(checkOutput(level)),
 			)
 
 			t.Setenv("TREEFMT_ON_UNMATCHED", levelStr)
@@ -107,15 +107,15 @@ func TestOnUnmatched(t *testing.T) {
 			treefmt(t,
 				withArgs("-vv"),
 				withNoError(t),
-				withOutput(checkOutput(level)),
+				withStderr(checkOutput(level)),
 			)
 		})
 	}
 
 	t.Run("invalid", func(t *testing.T) {
 		// test bad value
-		errorFn := func(arg string) func(err error) {
-			return func(err error) {
+		errorFn := func(arg string) func(as *require.Assertions, err error) {
+			return func(as *require.Assertions, err error) {
 				as.ErrorContains(err, fmt.Sprintf(`invalid level: "%s"`, arg))
 			}
 		}
@@ -131,6 +131,33 @@ func TestOnUnmatched(t *testing.T) {
 	})
 }
 
+func TestQuiet(t *testing.T) {
+	as := require.New(t)
+	tempDir := test.TempExamples(t)
+
+	test.ChangeWorkDir(t, tempDir)
+
+	// allow missing formatter
+	t.Setenv("TREEFMT_ALLOW_MISSING_FORMATTER", "true")
+
+	noOutput := func(out []byte) {
+		as.Empty(out)
+	}
+
+	treefmt(t, withArgs("-q"), withNoError(t), withStdout(noOutput), withStderr(noOutput))
+	treefmt(t, withArgs("--quiet"), withNoError(t), withStdout(noOutput), withStderr(noOutput))
+
+	t.Setenv("TREEFMT_QUIET", "true")
+	treefmt(t, withNoError(t), withStdout(noOutput), withStderr(noOutput))
+
+	t.Setenv("TREEFMT_ALLOW_MISSING_FORMATTER", "false")
+
+	// check it doesn't suppress errors
+	treefmt(t, withError(func(as *require.Assertions, err error) {
+		as.ErrorContains(err, "error looking up 'foo-fmt'")
+	}))
+}
+
 func TestCpuProfile(t *testing.T) {
 	as := require.New(t)
 	tempDir := test.TempExamples(t)
@@ -156,8 +183,6 @@ func TestCpuProfile(t *testing.T) {
 }
 
 func TestAllowMissingFormatter(t *testing.T) {
-	as := require.New(t)
-
 	tempDir := test.TempExamples(t)
 	configPath := filepath.Join(tempDir, "treefmt.toml")
 
@@ -173,7 +198,7 @@ func TestAllowMissingFormatter(t *testing.T) {
 
 	t.Run("default", func(t *testing.T) {
 		treefmt(t,
-			withError(func(err error) {
+			withError(func(as *require.Assertions, err error) {
 				as.ErrorIs(err, format.ErrCommandNotFound)
 			}),
 		)
@@ -184,7 +209,7 @@ func TestAllowMissingFormatter(t *testing.T) {
 			withArgs("--allow-missing-formatter"),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   0,
 				stats.Formatted: 0,
 				stats.Changed:   0,
@@ -199,15 +224,13 @@ func TestAllowMissingFormatter(t *testing.T) {
 }
 
 func TestSpecifyingFormatters(t *testing.T) {
-	as := require.New(t)
-
 	// we use the test formatter to append some whitespace
 	cfg := &config.Config{
 		FormatterConfigs: map[string]*config.Formatter{
-			"elm": {
+			"rust": {
 				Command:  "test-fmt-append",
 				Options:  []string{"   "},
-				Includes: []string{"*.elm"},
+				Includes: []string{"*.rs"},
 			},
 			"nix": {
 				Command:  "test-fmt-append",
@@ -233,7 +256,7 @@ func TestSpecifyingFormatters(t *testing.T) {
 			withNoError(t),
 			withModtimeBump(tempDir, time.Second),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   3,
 				stats.Formatted: 3,
 				stats.Changed:   3,
@@ -243,11 +266,11 @@ func TestSpecifyingFormatters(t *testing.T) {
 
 	t.Run("args", func(t *testing.T) {
 		treefmt(t,
-			withArgs("--formatters", "elm,nix"),
+			withArgs("--formatters", "rust,nix"),
 			withModtimeBump(tempDir, time.Second),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   2,
 				stats.Formatted: 2,
 				stats.Changed:   2,
@@ -259,7 +282,7 @@ func TestSpecifyingFormatters(t *testing.T) {
 			withModtimeBump(tempDir, time.Second),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   2,
 				stats.Formatted: 2,
 				stats.Changed:   2,
@@ -271,7 +294,7 @@ func TestSpecifyingFormatters(t *testing.T) {
 			withModtimeBump(tempDir, time.Second),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   1,
 				stats.Formatted: 1,
 				stats.Changed:   1,
@@ -281,8 +304,8 @@ func TestSpecifyingFormatters(t *testing.T) {
 		// bad name
 		treefmt(t,
 			withArgs("--formatters", "foo"),
-			withError(func(err error) {
-				as.Errorf(err, "formatter not found in config: foo")
+			withError(func(as *require.Assertions, err error) {
+				as.ErrorContains(err, "formatter foo not found in config")
 			}),
 		)
 	})
@@ -294,7 +317,7 @@ func TestSpecifyingFormatters(t *testing.T) {
 			withNoError(t),
 			withModtimeBump(tempDir, time.Second),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   2,
 				stats.Formatted: 2,
 				stats.Changed:   2,
@@ -304,11 +327,49 @@ func TestSpecifyingFormatters(t *testing.T) {
 		t.Setenv("TREEFMT_FORMATTERS", "bar,foo")
 
 		treefmt(t,
-			withError(func(err error) {
-				as.Errorf(err, "formatter not found in config: bar")
+			withError(func(as *require.Assertions, err error) {
+				as.ErrorContains(err, "formatter bar not found in config")
 			}),
 		)
 	})
+
+	t.Run("bad names", func(t *testing.T) {
+		for _, name := range []string{"foo$", "/bar", "baz%"} {
+			treefmt(t,
+				withArgs("--formatters", name),
+				withError(func(as *require.Assertions, err error) {
+					as.ErrorContains(err, fmt.Sprintf("formatter name %q is invalid", name))
+				}),
+			)
+
+			t.Setenv("TREEFMT_FORMATTERS", name)
+
+			treefmt(t,
+				withError(func(as *require.Assertions, err error) {
+					as.ErrorContains(err, fmt.Sprintf("formatter name %q is invalid", name))
+				}),
+			)
+
+			t.Setenv("TREEFMT_FORMATTERS", "")
+
+			cfg.FormatterConfigs[name] = &config.Formatter{
+				Command:  "echo",
+				Includes: []string{"*"},
+			}
+
+			test.WriteConfig(t, configPath, cfg)
+
+			treefmt(t,
+				withError(func(as *require.Assertions, err error) {
+					as.ErrorContains(err, fmt.Sprintf("formatter name %q is invalid", name))
+				}),
+			)
+
+			delete(cfg.FormatterConfigs, name)
+
+			test.WriteConfig(t, configPath, cfg)
+		}
+	})
 }
 
 func TestIncludesAndExcludes(t *testing.T) {
@@ -331,9 +392,9 @@ func TestIncludesAndExcludes(t *testing.T) {
 		withConfig(configPath, cfg),
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 33,
-			stats.Matched:   33,
-			stats.Formatted: 33,
+			stats.Traversed: 31,
+			stats.Matched:   31,
+			stats.Formatted: 31,
 			stats.Changed:   0,
 		}),
 	)
@@ -346,9 +407,9 @@ func TestIncludesAndExcludes(t *testing.T) {
 		withConfig(configPath, cfg),
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 33,
-			stats.Matched:   32,
-			stats.Formatted: 32,
+			stats.Traversed: 31,
+			stats.Matched:   30,
+			stats.Formatted: 30,
 			stats.Changed:   0,
 		}),
 	)
@@ -361,9 +422,9 @@ func TestIncludesAndExcludes(t *testing.T) {
 		withConfig(configPath, cfg),
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 33,
-			stats.Matched:   26,
-			stats.Formatted: 26,
+			stats.Traversed: 31,
+			stats.Matched:   24,
+			stats.Formatted: 24,
 			stats.Changed:   0,
 		}),
 	)
@@ -378,9 +439,9 @@ func TestIncludesAndExcludes(t *testing.T) {
 		withConfig(configPath, cfg),
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 33,
-			stats.Matched:   24,
-			stats.Formatted: 24,
+			stats.Traversed: 31,
+			stats.Matched:   22,
+			stats.Formatted: 22,
 			stats.Changed:   0,
 		}),
 	)
@@ -393,24 +454,24 @@ func TestIncludesAndExcludes(t *testing.T) {
 		withConfig(configPath, cfg),
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 33,
-			stats.Matched:   23,
-			stats.Formatted: 23,
+			stats.Traversed: 31,
+			stats.Matched:   21,
+			stats.Formatted: 21,
 			stats.Changed:   0,
 		}),
 	)
 
 	t.Setenv("TREEFMT_FORMATTER_ECHO_EXCLUDES", "") // reset
 
-	// adjust the includes for echo to only include elm files
-	echo.Includes = []string{"*.elm"}
+	// adjust the includes for echo to only include rust files
+	echo.Includes = []string{"*.rs"}
 
 	treefmt(t,
 		withArgs("-c"),
 		withConfig(configPath, cfg),
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 33,
+			stats.Traversed: 31,
 			stats.Matched:   1,
 			stats.Formatted: 1,
 			stats.Changed:   0,
@@ -418,14 +479,14 @@ func TestIncludesAndExcludes(t *testing.T) {
 	)
 
 	// add js files to echo formatter via env
-	t.Setenv("TREEFMT_FORMATTER_ECHO_INCLUDES", "*.elm,*.js")
+	t.Setenv("TREEFMT_FORMATTER_ECHO_INCLUDES", "*.rs,*.js")
 
 	treefmt(t,
 		withArgs("-c"),
 		withConfig(configPath, cfg),
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 33,
+			stats.Traversed: 31,
 			stats.Matched:   2,
 			stats.Formatted: 2,
 			stats.Changed:   0,
@@ -467,9 +528,9 @@ func TestConfigFile(t *testing.T) {
 				withArgs("--config-file", configPath, "--tree-root", tempDir),
 				withNoError(t),
 				withStats(t, map[stats.Type]int{
-					stats.Traversed: 33,
-					stats.Matched:   33,
-					stats.Formatted: 33,
+					stats.Traversed: 31,
+					stats.Matched:   31,
+					stats.Formatted: 31,
 					stats.Changed:   0,
 				}),
 			)
@@ -498,8 +559,8 @@ func TestConfigFile(t *testing.T) {
 				}),
 				withNoError(t),
 				withStats(t, map[stats.Type]int{
-					stats.Traversed: 33,
-					stats.Matched:   33,
+					stats.Traversed: 31,
+					stats.Matched:   31,
 					stats.Formatted: 0,
 					stats.Changed:   0,
 				}),
@@ -514,7 +575,7 @@ func TestConfigFile(t *testing.T) {
 				withEnv(map[string]string{
 					"PRJ_ROOT": configSubDir,
 				}),
-				withError(func(err error) {
+				withError(func(as *require.Assertions, err error) {
 					as.ErrorContains(err, "failed to find treefmt config file")
 				}),
 			)
@@ -523,8 +584,6 @@ func TestConfigFile(t *testing.T) {
 }
 
 func TestCache(t *testing.T) {
-	as := require.New(t)
-
 	tempDir := test.TempExamples(t)
 	configPath := filepath.Join(tempDir, "treefmt.toml")
 
@@ -547,10 +606,10 @@ func TestCache(t *testing.T) {
 	treefmt(t,
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 33,
-			stats.Matched:   33,
-			stats.Formatted: 33,
-			stats.Changed:   33,
+			stats.Traversed: 31,
+			stats.Matched:   31,
+			stats.Formatted: 31,
+			stats.Changed:   31,
 		}),
 	)
 
@@ -558,8 +617,8 @@ func TestCache(t *testing.T) {
 	treefmt(t,
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 33,
-			stats.Matched:   33,
+			stats.Traversed: 31,
+			stats.Matched:   31,
 			stats.Formatted: 0,
 			stats.Changed:   0,
 		}),
@@ -570,10 +629,10 @@ func TestCache(t *testing.T) {
 		withArgs("-c"),
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 33,
-			stats.Matched:   33,
-			stats.Formatted: 33,
-			stats.Changed:   33,
+			stats.Traversed: 31,
+			stats.Matched:   31,
+			stats.Formatted: 31,
+			stats.Changed:   31,
 		}),
 	)
 
@@ -581,8 +640,8 @@ func TestCache(t *testing.T) {
 	treefmt(t,
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 33,
-			stats.Matched:   33,
+			stats.Traversed: 31,
+			stats.Matched:   31,
 			stats.Formatted: 0,
 			stats.Changed:   0,
 		}),
@@ -593,10 +652,10 @@ func TestCache(t *testing.T) {
 		withNoError(t),
 		withModtimeBump(tempDir, time.Second),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 33,
-			stats.Matched:   33,
-			stats.Formatted: 33,
-			stats.Changed:   33,
+			stats.Traversed: 31,
+			stats.Matched:   31,
+			stats.Formatted: 31,
+			stats.Changed:   31,
 		}),
 	)
 
@@ -605,10 +664,10 @@ func TestCache(t *testing.T) {
 		withArgs("--no-cache"),
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 33,
-			stats.Matched:   33,
-			stats.Formatted: 33,
-			stats.Changed:   33,
+			stats.Traversed: 31,
+			stats.Matched:   31,
+			stats.Formatted: 31,
+			stats.Changed:   31,
 		}),
 	)
 
@@ -630,11 +689,11 @@ func TestCache(t *testing.T) {
 	// running should match but not format anything
 
 	treefmt(t,
-		withError(func(err error) {
+		withError(func(as *require.Assertions, err error) {
 			as.ErrorIs(err, format.ErrFormattingFailures)
 		}),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 33,
+			stats.Traversed: 31,
 			stats.Matched:   6,
 			stats.Formatted: 0,
 			stats.Changed:   0,
@@ -643,11 +702,11 @@ func TestCache(t *testing.T) {
 
 	// running again should provide the same result
 	treefmt(t,
-		withError(func(err error) {
+		withError(func(as *require.Assertions, err error) {
 			as.ErrorIs(err, format.ErrFormattingFailures)
 		}),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 33,
+			stats.Traversed: 31,
 			stats.Matched:   6,
 			stats.Formatted: 0,
 			stats.Changed:   0,
@@ -667,7 +726,7 @@ func TestCache(t *testing.T) {
 	treefmt(t,
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 33,
+			stats.Traversed: 31,
 			stats.Matched:   6,
 			stats.Formatted: 6,
 			stats.Changed:   6,
@@ -694,6 +753,7 @@ func TestChangeWorkingDirectory(t *testing.T) {
 		as.NoError(err)
 
 		t.Cleanup(func() {
+			//nolint:usetesting
 			// return to the previous working directory
 			as.NoError(os.Chdir(cwd))
 		})
@@ -701,16 +761,18 @@ func TestChangeWorkingDirectory(t *testing.T) {
 		tempDir := test.TempExamples(t)
 		configPath := filepath.Join(tempDir, "treefmt.toml")
 
+		//nolint:usetesting
 		// change to an empty temp dir and try running without specifying a working directory
 		as.NoError(os.Chdir(t.TempDir()))
 
 		treefmt(t,
 			withConfig(configPath, cfg),
-			withError(func(err error) {
+			withError(func(as *require.Assertions, err error) {
 				as.ErrorContains(err, "failed to find treefmt config file")
 			}),
 		)
 
+		//nolint:usetesting
 		// now change to the examples temp directory
 		as.NoError(os.Chdir(tempDir), "failed to change to temp directory")
 
@@ -718,7 +780,7 @@ func TestChangeWorkingDirectory(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 			}),
 		)
 	})
@@ -731,6 +793,7 @@ func TestChangeWorkingDirectory(t *testing.T) {
 			as.NoError(err)
 
 			t.Cleanup(func() {
+				//nolint:usetesting
 				// return to the previous working directory
 				as.NoError(os.Chdir(cwd))
 			})
@@ -754,7 +817,7 @@ func TestChangeWorkingDirectory(t *testing.T) {
 				withConfig(configPath, cfg),
 				withNoError(t),
 				withStats(t, map[stats.Type]int{
-					stats.Traversed: 33,
+					stats.Traversed: 31,
 				}),
 			)
 		})
@@ -777,8 +840,6 @@ func TestChangeWorkingDirectory(t *testing.T) {
 }
 
 func TestFailOnChange(t *testing.T) {
-	as := require.New(t)
-
 	t.Run("change size", func(t *testing.T) {
 		tempDir := test.TempExamples(t)
 		configPath := filepath.Join(tempDir, "treefmt.toml")
@@ -792,34 +853,34 @@ func TestFailOnChange(t *testing.T) {
 					// an arbitrary value to a list of files
 					Command:  "test-fmt-append",
 					Options:  []string{"hello"},
-					Includes: []string{"elm/*"},
+					Includes: []string{"rust/*"},
 				},
 			},
 		}
 
-		// running with a cold cache, we should see the elm files being formatted, resulting in changes, which should
+		// running with a cold cache, we should see the rust files being formatted, resulting in changes, which should
 		// trigger an error
 		treefmt(t,
 			withArgs("--fail-on-change"),
 			withConfig(configPath, cfg),
-			withError(func(err error) {
+			withError(func(as *require.Assertions, err error) {
 				as.ErrorIs(err, formatCmd.ErrFailOnChange)
 			}),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   2,
 				stats.Formatted: 2,
 				stats.Changed:   2,
 			}),
 		)
 
-		// running with a hot cache, we should see matches for the elm files, but no attempt to format them as the
+		// running with a hot cache, we should see matches for the rust files, but no attempt to format them as the
 		// underlying files have not changed since we last ran
 		treefmt(t,
 			withArgs("--fail-on-change"),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   2,
 				stats.Formatted: 0,
 				stats.Changed:   0,
@@ -863,11 +924,11 @@ func TestFailOnChange(t *testing.T) {
 					},
 				}
 			}),
-			withError(func(err error) {
+			withError(func(as *require.Assertions, err error) {
 				as.ErrorIs(err, formatCmd.ErrFailOnChange)
 			}),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   7,
 				stats.Formatted: 7,
 				stats.Changed:   7,
@@ -880,7 +941,7 @@ func TestFailOnChange(t *testing.T) {
 			withArgs("--fail-on-change"),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   7,
 				stats.Formatted: 0,
 				stats.Changed:   0,
@@ -918,7 +979,7 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   8,
 				stats.Formatted: 8,
 				stats.Changed:   6,
@@ -932,7 +993,7 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   8,
 				stats.Formatted: 6,
 				stats.Changed:   6,
@@ -943,7 +1004,7 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   8,
 				stats.Formatted: 0,
 				stats.Changed:   0,
@@ -957,7 +1018,7 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   8,
 				stats.Formatted: 6,
 				stats.Changed:   0, // echo doesn't affect the files so no changes expected
@@ -968,7 +1029,7 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   8,
 				stats.Formatted: 0,
 				stats.Changed:   0,
@@ -983,7 +1044,7 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   6,
 				stats.Formatted: 0,
 				stats.Changed:   0,
@@ -998,7 +1059,7 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   5,
 				stats.Formatted: 0,
 				stats.Changed:   0,
@@ -1032,10 +1093,10 @@ func TestCacheBusting(t *testing.T) {
 					Command:  "echo", // this is non-destructive, will match but cause no changes
 					Includes: []string{"*.py"},
 				},
-				"elm": {
+				"rust": {
 					Command:  "test-fmt-append",
 					Options:  []string{"   "},
-					Includes: []string{"*.elm"},
+					Includes: []string{"*.rs"},
 				},
 			},
 		}
@@ -1045,22 +1106,22 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 34,
+				stats.Traversed: 32,
 				stats.Matched:   3,
 				stats.Formatted: 3,
 				stats.Changed:   1,
 			}))
 
-		// tweak mod time of elm formatter
+		// tweak mod time of rust formatter
 		newTime := time.Now().Add(-time.Minute)
 		as.NoError(os.Chtimes(scriptPath, newTime, newTime))
 
-		// cache entries for elm files should be invalidated
+		// cache entries for rust files should be invalidated
 		treefmt(t,
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 34,
+				stats.Traversed: 32,
 				stats.Matched:   3,
 				stats.Formatted: 1,
 				stats.Changed:   1,
@@ -1071,27 +1132,27 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 34,
+				stats.Traversed: 32,
 				stats.Matched:   3,
 				stats.Formatted: 0,
 				stats.Changed:   0,
 			}),
 		)
 
-		// tweak the size of elm formatter
+		// tweak the size of rust formatter
 		formatter, err := os.OpenFile(scriptPath, os.O_WRONLY|os.O_APPEND, 0o755)
-		as.NoError(err, "failed to open elm formatter")
+		as.NoError(err, "failed to open rust formatter")
 
 		_, err = formatter.WriteString(" ") // add some whitespace
-		as.NoError(err, "failed to append to elm formatter")
-		as.NoError(formatter.Close(), "failed to close elm formatter")
+		as.NoError(err, "failed to append to rust formatter")
+		as.NoError(formatter.Close(), "failed to close rust formatter")
 
-		// cache entries for elm files should be invalidated
+		// cache entries for rust files should be invalidated
 		treefmt(t,
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 34,
+				stats.Traversed: 32,
 				stats.Matched:   3,
 				stats.Formatted: 1,
 				stats.Changed:   1,
@@ -1102,7 +1163,7 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 34,
+				stats.Traversed: 32,
 				stats.Matched:   3,
 				stats.Formatted: 0,
 				stats.Changed:   0,
@@ -1131,7 +1192,7 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   2,
 				stats.Formatted: 2,
 				stats.Changed:   2,
@@ -1143,7 +1204,7 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   2,
 				stats.Formatted: 0,
 				stats.Changed:   0,
@@ -1151,18 +1212,18 @@ func TestCacheBusting(t *testing.T) {
 		)
 
 		// add a formatter
-		cfg.FormatterConfigs["elm"] = &config.Formatter{
+		cfg.FormatterConfigs["rust"] = &config.Formatter{
 			Command:  "test-fmt-append",
 			Options:  []string{"   "},
-			Includes: []string{"*.elm"},
+			Includes: []string{"*.rs"},
 		}
 
-		// only the elm files should be formatted
+		// only the rust files should be formatted
 		treefmt(t,
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   3,
 				stats.Formatted: 1,
 				stats.Changed:   1,
@@ -1181,7 +1242,7 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   3,
 				stats.Formatted: 2,
 				stats.Changed:   2,
@@ -1193,7 +1254,7 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   3,
 				stats.Formatted: 0,
 				stats.Changed:   0,
@@ -1209,7 +1270,7 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   3,
 				stats.Formatted: 2,
 				stats.Changed:   2,
@@ -1221,7 +1282,7 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   3,
 				stats.Formatted: 0,
 				stats.Changed:   0,
@@ -1236,7 +1297,7 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   3,
 				stats.Formatted: 2,
 				stats.Changed:   2,
@@ -1248,15 +1309,15 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   3,
 				stats.Formatted: 0,
 				stats.Changed:   0,
 			}),
 		)
 
-		// remove the elm formatter
-		delete(cfg.FormatterConfigs, "elm")
+		// remove the rust formatter
+		delete(cfg.FormatterConfigs, "rust")
 
 		// only python files should match, but no formatting should occur as not formatting signatures have been
 		// affected
@@ -1264,7 +1325,7 @@ func TestCacheBusting(t *testing.T) {
 			withConfig(configPath, cfg),
 			withNoError(t),
 			withStats(t, map[stats.Type]int{
-				stats.Traversed: 33,
+				stats.Traversed: 31,
 				stats.Matched:   2,
 				stats.Formatted: 0,
 				stats.Changed:   0,
@@ -1298,11 +1359,15 @@ func TestGit(t *testing.T) {
 	as.NoError(gitCmd.Run(), "failed to init git repository")
 
 	// run before adding anything to the index
+	// we should pick up untracked files since we use `git ls-files -o`
 	treefmt(t,
 		withConfig(configPath, cfg),
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 0,
+			stats.Traversed: 31,
+			stats.Matched:   31,
+			stats.Formatted: 31,
+			stats.Changed:   0,
 		}),
 	)
 
@@ -1314,16 +1379,15 @@ func TestGit(t *testing.T) {
 		withConfig(configPath, cfg),
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 33,
-			stats.Matched:   33,
-			stats.Formatted: 33,
+			stats.Traversed: 31,
+			stats.Matched:   31,
+			stats.Formatted: 0,
 			stats.Changed:   0,
 		}),
 	)
 
-	// remove python directory from the index
-	gitCmd = exec.Command("git", "rm", "--cached", "python/*")
-	as.NoError(gitCmd.Run(), "failed to remove python directory from the index")
+	// remove python directory
+	as.NoError(os.RemoveAll(filepath.Join(tempDir, "python")), "failed to remove python directory")
 
 	// we should traverse and match against fewer files, but no formatting should occur as no formatting signatures
 	// are impacted
@@ -1331,8 +1395,8 @@ func TestGit(t *testing.T) {
 		withConfig(configPath, cfg),
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 30,
-			stats.Matched:   30,
+			stats.Traversed: 28,
+			stats.Matched:   28,
 			stats.Formatted: 0,
 			stats.Changed:   0,
 		}),
@@ -1343,16 +1407,16 @@ func TestGit(t *testing.T) {
 
 	// walk with filesystem instead of with git
 	// the .git folder contains 50 additional files
-	// when added to the 32 we started with (34 minus nixpkgs.toml which we removed from the filesystem), we should
+	// when added to the 30 we started with (34 minus nixpkgs.toml which we removed from the filesystem), we should
 	// traverse 82 files.
 	treefmt(t,
 		withArgs("--walk", "filesystem"),
 		withConfig(configPath, cfg),
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 82,
-			stats.Matched:   82,
-			stats.Formatted: 50, // the echo formatter should only be applied to the new files
+			stats.Traversed: 76,
+			stats.Matched:   76,
+			stats.Formatted: 49, // the echo formatter should only be applied to the new files
 			stats.Changed:   0,
 		}),
 	)
@@ -1401,17 +1465,17 @@ func TestGit(t *testing.T) {
 	treefmt(t,
 		withArgs("-C", tempDir, "haskell", "foo"),
 		withConfig(configPath, cfg),
-		withError(func(err error) {
-			as.ErrorContains(err, "path foo not found")
+		withError(func(as *require.Assertions, err error) {
+			as.ErrorContains(err, "foo not found")
 		}),
 	)
 
-	// try with a path not in the git index, e.g. it is skipped
+	// try with a path not in the git index
 	_, err := os.Create(filepath.Join(tempDir, "foo.txt"))
 	as.NoError(err)
 
 	treefmt(t,
-		withArgs("haskell", "foo.txt"),
+		withArgs("haskell", "foo.txt", "-vv"),
 		withConfig(configPath, cfg),
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
@@ -1455,6 +1519,7 @@ func TestPathsArg(t *testing.T) {
 	as.NoError(err)
 
 	t.Cleanup(func() {
+		//nolint:usetesting
 		// return to the previous working directory
 		as.NoError(os.Chdir(cwd))
 	})
@@ -1472,6 +1537,7 @@ func TestPathsArg(t *testing.T) {
 	externalFile, err := os.Create(filepath.Join(tempDir, "outside_tree.go"))
 	as.NoError(err)
 
+	//nolint:usetesting
 	// change working directory to project root
 	as.NoError(os.Chdir(treeRoot))
 
@@ -1491,16 +1557,16 @@ func TestPathsArg(t *testing.T) {
 	treefmt(t,
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
-			stats.Traversed: 33,
-			stats.Matched:   33,
-			stats.Formatted: 33,
+			stats.Traversed: 31,
+			stats.Matched:   31,
+			stats.Formatted: 31,
 			stats.Changed:   0,
 		}),
 	)
 
 	// specify some explicit paths
 	treefmt(t,
-		withArgs("elm/elm.json", "haskell/Nested/Foo.hs"),
+		withArgs("rust/src/main.rs", "haskell/Nested/Foo.hs"),
 		withNoError(t),
 		withStats(t, map[stats.Type]int{
 			stats.Traversed: 2,
@@ -1511,7 +1577,7 @@ func TestPathsArg(t *testing.T) {
 	)
 
 	// specify an absolute path
-	absoluteInternalPath, err := filepath.Abs("elm/elm.json")
+	absoluteInternalPath, err := filepath.Abs("rust/src/main.rs")
 	as.NoError(err)
 
 	treefmt(t,
@@ -1527,9 +1593,9 @@ func TestPathsArg(t *testing.T) {
 
 	// specify a bad path
 	treefmt(t,
-		withArgs("elm/elm.json", "haskell/Nested/Bar.hs"),
-		withError(func(err error) {
-			as.Errorf(err, "path haskell/Nested/Bar.hs not found")
+		withArgs("rust/src/main.rs", "haskell/Nested/Bar.hs"),
+		withError(func(as *require.Assertions, err error) {
+			as.ErrorContains(err, "Bar.hs not found")
 		}),
 	)
 
@@ -1540,19 +1606,19 @@ func TestPathsArg(t *testing.T) {
 
 	treefmt(t,
 		withArgs(absoluteExternalPath),
-		withError(func(err error) {
-			as.Errorf(err, "path %s not found within the tree root", absoluteExternalPath)
+		withError(func(as *require.Assertions, err error) {
+			as.ErrorContains(err, fmt.Sprintf("path %s not inside the tree root", absoluteExternalPath))
 		}),
 	)
 
 	// specify a relative path outside the tree root
 	relativeExternalPath := "../outside_tree.go"
-	as.FileExists(relativeExternalPath, "exernal file must exist")
+	as.FileExists(relativeExternalPath, "external file must exist")
 
 	treefmt(t,
 		withArgs(relativeExternalPath),
-		withError(func(err error) {
-			as.Errorf(err, "path %s not found within the tree root", relativeExternalPath)
+		withError(func(as *require.Assertions, err error) {
+			as.ErrorContains(err, fmt.Sprintf("path %s not inside the tree root", relativeExternalPath))
 		}),
 	)
 }
@@ -1580,10 +1646,10 @@ func TestStdin(t *testing.T) {
 	// we get an error about the missing filename parameter.
 	treefmt(t,
 		withArgs("--stdin"),
-		withError(func(err error) {
+		withError(func(as *require.Assertions, err error) {
 			as.EqualError(err, "exactly one path should be specified when using the --stdin flag")
 		}),
-		withOutput(func(out []byte) {
+		withStderr(func(out []byte) {
 			as.Equal("Error: exactly one path should be specified when using the --stdin flag\n", string(out))
 		}),
 	)
@@ -1600,7 +1666,7 @@ func TestStdin(t *testing.T) {
 			stats.Formatted: 1,
 			stats.Changed:   1,
 		}),
-		withOutput(func(out []byte) {
+		withStdout(func(out []byte) {
 			as.Equal(`{ ...}: "hello"
 `, string(out))
 		}),
@@ -1613,10 +1679,10 @@ func TestStdin(t *testing.T) {
 
 	treefmt(t,
 		withArgs("--stdin", "../test.nix"),
-		withError(func(err error) {
-			as.Errorf(err, "path ../test.nix not inside the tree root %s", tempDir)
+		withError(func(as *require.Assertions, err error) {
+			as.ErrorContains(err, "path ../test.nix not inside the tree root "+tempDir)
 		}),
-		withOutput(func(out []byte) {
+		withStderr(func(out []byte) {
 			as.Contains(string(out), "Error: path ../test.nix not inside the tree root")
 		}),
 	)
@@ -1639,7 +1705,7 @@ func TestStdin(t *testing.T) {
 			stats.Formatted: 1,
 			stats.Changed:   1,
 		}),
-		withOutput(func(out []byte) {
+		withStdout(func(out []byte) {
 			as.Equal(`| col1   | col2      |
 | ------ | --------- |
 | nice   | fits      |
@@ -1747,8 +1813,9 @@ func TestRunInSubdir(t *testing.T) {
 			err = os.Symlink(echoPath, echoRel)
 			as.NoError(err)
 
+			//nolint:usetesting
 			// change working directory to subdirectory
-			as.NoError(os.Chdir(filepath.Join(tempDir, "elm")))
+			as.NoError(os.Chdir(filepath.Join(tempDir, "go")))
 
 			// basic config
 			cfg := &config.Config{
@@ -1766,9 +1833,9 @@ func TestRunInSubdir(t *testing.T) {
 			treefmt(t,
 				withNoError(t),
 				withStats(t, map[stats.Type]int{
-					stats.Traversed: 33,
-					stats.Matched:   33,
-					stats.Formatted: 33,
+					stats.Traversed: 31,
+					stats.Matched:   31,
+					stats.Formatted: 31,
 					stats.Changed:   0,
 				}),
 			)
@@ -1776,15 +1843,15 @@ func TestRunInSubdir(t *testing.T) {
 			// specify some explicit paths, relative to the tree root
 			// this should not work, as we're in a subdirectory
 			treefmt(t,
-				withArgs("-c", "elm/elm.json", "haskell/Nested/Foo.hs"),
-				withError(func(err error) {
-					as.ErrorContains(err, "path elm/elm.json not found")
+				withArgs("-c", "go/main.go", "haskell/Nested/Foo.hs"),
+				withError(func(as *require.Assertions, err error) {
+					as.ErrorContains(err, "go/main.go not found")
 				}),
 			)
 
 			// specify some explicit paths, relative to the current directory
 			treefmt(t,
-				withArgs("-c", "elm.json", "../haskell/Nested/Foo.hs"),
+				withArgs("-c", "main.go", "../haskell/Nested/Foo.hs"),
 				withNoError(t),
 				withStats(t, map[stats.Type]int{
 					stats.Traversed: 2,
@@ -1806,8 +1873,10 @@ type options struct {
 		value *config.Config
 	}
 
-	assertOut   func([]byte)
-	assertError func(error)
+	assertStdout func([]byte)
+	assertStderr func([]byte)
+
+	assertError func(*require.Assertions, error)
 	assertStats func(*stats.Stats)
 
 	bump struct {
@@ -1857,7 +1926,7 @@ func withStats(t *testing.T, expected map[stats.Type]int) option {
 	}
 }
 
-func withError(fn func(error)) option {
+func withError(fn func(*require.Assertions, error)) option {
 	return func(o *options) {
 		o.assertError = fn
 	}
@@ -1867,15 +1936,21 @@ func withNoError(t *testing.T) option {
 	t.Helper()
 
 	return func(o *options) {
-		o.assertError = func(err error) {
-			require.NoError(t, err)
+		o.assertError = func(as *require.Assertions, err error) {
+			as.NoError(err)
 		}
 	}
 }
 
-func withOutput(fn func([]byte)) option {
+func withStdout(fn func([]byte)) option {
 	return func(o *options) {
-		o.assertOut = fn
+		o.assertStdout = fn
+	}
+}
+
+func withStderr(fn func([]byte)) option {
+	return func(o *options) {
+		o.assertStderr = fn
 	}
 }
 
@@ -1893,6 +1968,8 @@ func treefmt(
 ) {
 	t.Helper()
 
+	as := require.New(t)
+
 	// build options
 	opts := &options{}
 	for _, option := range opt {
@@ -1931,17 +2008,19 @@ func treefmt(
 	t.Logf("treefmt %s", strings.Join(args, " "))
 
 	tempDir := t.TempDir()
-	tempOut := test.TempFile(t, tempDir, "combined_output", nil)
+
+	tempStdout := test.TempFile(t, tempDir, "stdout", nil)
+	tempStderr := test.TempFile(t, tempDir, "stderr", nil)
 
 	// capture standard outputs before swapping them
 	stdout := os.Stdout
 	stderr := os.Stderr
 
 	// swap them temporarily
-	os.Stdout = tempOut
-	os.Stderr = tempOut
+	os.Stdout = tempStdout
+	os.Stderr = tempStderr
 
-	log.SetOutput(tempOut)
+	log.SetOutput(tempStdout)
 
 	defer func() {
 		// swap outputs back
@@ -1954,33 +2033,52 @@ func treefmt(
 	root, statz := cmd.NewRoot()
 
 	root.SetArgs(args)
-	root.SetOut(tempOut)
-	root.SetErr(tempOut)
+	root.SetOut(tempStdout)
+	root.SetErr(tempStderr)
 
 	// execute the command
 	cmdErr := root.Execute()
 
-	// reset and read the temporary output
-	if _, resetErr := tempOut.Seek(0, 0); resetErr != nil {
+	// reset and read the temporary outputs
+	if _, resetErr := tempStdout.Seek(0, 0); resetErr != nil {
 		t.Fatal(fmt.Errorf("failed to reset temp output for reading: %w", resetErr))
 	}
 
-	out, readErr := io.ReadAll(tempOut)
+	if _, resetErr := tempStderr.Seek(0, 0); resetErr != nil {
+		t.Fatal(fmt.Errorf("failed to reset temp output for reading: %w", resetErr))
+	}
+
+	// read back stderr and validate
+	out, readErr := io.ReadAll(tempStderr)
 	if readErr != nil {
-		t.Fatal(fmt.Errorf("failed to read temp output: %w", readErr))
+		t.Fatal(fmt.Errorf("failed to read temp stderr: %w", readErr))
+	}
+
+	if opts.assertStderr != nil {
+		opts.assertStderr(out)
 	}
 
 	t.Log("\n" + string(out))
 
+	// read back stdout and validate
+	out, readErr = io.ReadAll(tempStdout)
+	if readErr != nil {
+		t.Fatal(fmt.Errorf("failed to read temp stdout: %w", readErr))
+	}
+
+	t.Log("\n" + string(out))
+
+	if opts.assertStdout != nil {
+		opts.assertStdout(out)
+	}
+
+	// assert other properties
+
 	if opts.assertStats != nil {
 		opts.assertStats(statz)
 	}
 
-	if opts.assertOut != nil {
-		opts.assertOut(out)
-	}
-
 	if opts.assertError != nil {
-		opts.assertError(cmdErr)
+		opts.assertError(as, cmdErr)
 	}
 }
