@@ -4,6 +4,7 @@ import (
 	"fmt"
 	"os"
 	"path/filepath"
+	"regexp"
 	"strings"
 
 	"github.com/numtide/treefmt/v2/walk"
@@ -22,6 +23,7 @@ type Config struct {
 	Formatters            []string `mapstructure:"formatters"              toml:"formatters,omitempty"`
 	NoCache               bool     `mapstructure:"no-cache"                toml:"-"` // not allowed in config
 	OnUnmatched           string   `mapstructure:"on-unmatched"            toml:"on-unmatched,omitempty"`
+	Quiet                 bool     `mapstructure:"quiet"                   toml:"-"` // not allowed in config
 	TreeRoot              string   `mapstructure:"tree-root"               toml:"tree-root,omitempty"`
 	TreeRootFile          string   `mapstructure:"tree-root-file"          toml:"tree-root-file,omitempty"`
 	Verbose               uint8    `mapstructure:"verbose"                 toml:"verbose,omitempty"`
@@ -89,7 +91,7 @@ func SetFlags(fs *pflag.FlagSet) {
 		"Ignore the evaluation cache entirely. Useful for CI. (env $TREEFMT_NO_CACHE)",
 	)
 	fs.StringP(
-		"on-unmatched", "u", "warn",
+		"on-unmatched", "u", "info",
 		"Log paths that did not match any formatters at the specified log level. Possible values are "+
 			"<debug|info|warn|error|fatal>. (env $TREEFMT_ON_UNMATCHED)",
 	)
@@ -110,6 +112,9 @@ func SetFlags(fs *pflag.FlagSet) {
 		"verbose", "v",
 		"Set the verbosity of logs e.g. -vv. (env $TREEFMT_VERBOSE)",
 	)
+	fs.BoolP(
+		"quiet", "q", false, "Disable all logs except errors. (env $TREEFMT_QUIET)",
+	)
 	fs.String(
 		"walk", "auto",
 		"The method used to traverse the files within the tree root. Currently supports "+
@@ -206,12 +211,32 @@ func FromViper(v *viper.Viper) (*Config, error) {
 		cfg.Excludes = cfg.Global.Excludes
 	}
 
+	// validate formatter names do not contain invalid characters
+
+	nameRegex := regexp.MustCompile("^[a-zA-Z0-9_-]+$")
+
+	for name := range cfg.FormatterConfigs {
+		if !nameRegex.MatchString(name) {
+			return nil, fmt.Errorf(
+				"formatter name %q is invalid, must be of the form %s",
+				name, nameRegex.String(),
+			)
+		}
+	}
+
 	// filter formatters based on provided names
 	if len(cfg.Formatters) > 0 {
 		filtered := make(map[string]*Formatter)
 
 		// check if the provided names exist in the config
 		for _, name := range cfg.Formatters {
+			if !nameRegex.MatchString(name) {
+				return nil, fmt.Errorf(
+					"formatter name %q is invalid, must be of the form %s",
+					name, nameRegex.String(),
+				)
+			}
+
 			formatterCfg, ok := cfg.FormatterConfigs[name]
 			if !ok {
 				return nil, fmt.Errorf("formatter %v not found in config", name)
