@@ -15,8 +15,6 @@ import (
 
 //nolint:gochecknoglobals
 var examplesPaths = []string{
-	"elm/elm.json",
-	"elm/src/Main.elm",
 	"emoji 🕰️/README.md",
 	"go/go.mod",
 	"go/main.go",
@@ -61,7 +59,7 @@ func TestFilesystemReader(t *testing.T) {
 	count := 0
 
 	for {
-		ctx, cancel := context.WithTimeout(context.Background(), 100*time.Millisecond)
+		ctx, cancel := context.WithTimeout(t.Context(), 100*time.Millisecond)
 
 		files := make([]*walk.File, 8)
 		n, err := r.Read(ctx, files)
@@ -79,8 +77,8 @@ func TestFilesystemReader(t *testing.T) {
 		}
 	}
 
-	as.Equal(33, count)
-	as.Equal(33, statz.Value(stats.Traversed))
+	as.Equal(31, count)
+	as.Equal(31, statz.Value(stats.Traversed))
 	as.Equal(0, statz.Value(stats.Matched))
 	as.Equal(0, statz.Value(stats.Formatted))
 	as.Equal(0, statz.Value(stats.Changed))
