@@ -29,12 +29,12 @@ func TestGitReader(t *testing.T) {
 	reader, err := walk.NewGitReader(tempDir, "", &statz)
 	as.NoError(err)
 
-	files := make([]*walk.File, 8)
-	ctx, cancel := context.WithTimeout(context.Background(), 100*time.Millisecond)
+	files := make([]*walk.File, 32)
+	ctx, cancel := context.WithTimeout(t.Context(), 100*time.Millisecond)
 	n, err := reader.Read(ctx, files)
 
 	cancel()
-	as.Equal(0, n)
+	as.Equal(31, n)
 	as.ErrorIs(err, io.EOF)
 
 	// add everything to the git index
@@ -42,13 +42,14 @@ func TestGitReader(t *testing.T) {
 	cmd.Dir = tempDir
 	as.NoError(cmd.Run(), "failed to add everything to the index")
 
+	statz = stats.New()
 	reader, err = walk.NewGitReader(tempDir, "", &statz)
 	as.NoError(err)
 
 	count := 0
 
 	for {
-		ctx, cancel := context.WithTimeout(context.Background(), 100*time.Millisecond)
+		ctx, cancel := context.WithTimeout(t.Context(), 100*time.Millisecond)
 
 		files := make([]*walk.File, 8)
 		n, err := reader.Read(ctx, files)
@@ -62,8 +63,8 @@ func TestGitReader(t *testing.T) {
 		}
 	}
 
-	as.Equal(33, count)
-	as.Equal(33, statz.Value(stats.Traversed))
+	as.Equal(31, count)
+	as.Equal(31, statz.Value(stats.Traversed))
 	as.Equal(0, statz.Value(stats.Matched))
 	as.Equal(0, statz.Value(stats.Formatted))
 	as.Equal(0, statz.Value(stats.Changed))
